@@ -44,9 +44,10 @@ use crate::parser::rislive::messages::ris_message::path_to_as_path;
 
 use crate::BgpElem;
 use std::net::IpAddr;
-use bgp_models::bgp::attributes::Community;
+use bgp_models::bgp::community::Community;
 use bgp_models::bgp::attributes::Origin::{EGP, IGP, INCOMPLETE};
-use bgp_models::network::NetworkPrefix;
+use bgp_models::bgp::MetaCommunity;
+use bgp_models::network::{Asn, NetworkPrefix};
 use ipnetwork::IpNetwork;
 use crate::parser::ElemType;
 
@@ -55,10 +56,10 @@ pub mod messages;
 
 // simple macro to make the code look a bit nicer
 macro_rules! unwrap_or_return {
-    ( $e:expr ) => {
+    ( $e:expr, $msg_string:expr ) => {
         match $e {
             Ok(x) => x,
-            Err(_) => return Err(ParserRisliveError::IncorrectJson),
+            Err(_) => return Err(ParserRisliveError::IncorrectJson($msg_string)),
         }
     }
 }
@@ -66,10 +67,12 @@ macro_rules! unwrap_or_return {
 /// This function parses one message and returns a result of a vector of [BgpElem]s or an error
 pub fn parse_ris_live_message(msg_str: &str) -> Result<Vec<BgpElem>, ParserRisliveError> {
 
+    let msg_string = msg_str.to_string();
+
     // parse RIS Live message to internal struct using serde.
     let msg: RisLiveMessage = match serde_json::from_str(msg_str) {
         Ok(m) => m,
-        Err(_e) => return Err(ParserRisliveError::IncorrectJson),
+        Err(_e) => return Err(ParserRisliveError::IncorrectJson(msg_string)),
     };
 
     match msg {
@@ -93,8 +96,8 @@ pub fn parse_ris_live_message(msg_str: &str) -> Result<Vec<BgpElem>, ParserRisli
                 } => {
                     let mut elems: Vec<BgpElem> = vec![];
 
-                    let peer_ip = unwrap_or_return!(ris_msg.peer.parse::<IpAddr>());
-                    let peer_asn = unwrap_or_return!(ris_msg.peer_asn.parse::<u32>());
+                    let peer_ip = unwrap_or_return!(ris_msg.peer.parse::<IpAddr>(), msg_string);
+                    let peer_asn: Asn = unwrap_or_return!(ris_msg.peer_asn.parse::<i32>(), msg_string).into();
 
                     // parse path
                     let as_path = match path{
@@ -106,9 +109,10 @@ pub fn parse_ris_live_message(msg_str: &str) -> Result<Vec<BgpElem>, ParserRisli
                     let communities = match community {
                         None => {None}
                         Some(cs) => {
-                            let mut comms: Vec<Community> = vec![];
+                            let mut comms: Vec<MetaCommunity> = vec![];
                             for c in cs {
-                                comms.push(Community::Custom(c.0,c.1));
+                                comms.push(
+                                    MetaCommunity::Community(Community::Custom((c.0 as i32).into(),c.1)));
                             }
                             Some(comms)
                         }
@@ -143,8 +147,8 @@ pub fn parse_ris_live_message(msg_str: &str) -> Result<Vec<BgpElem>, ParserRisli
                             if parts.len()!=2 {
                                 return Err(ParserRisliveError::ElemIncorrectAggregator(aggr_str))
                             }
-                            let asn = unwrap_or_return!(parts[0].to_owned().parse::<u32>());
-                            let ip = unwrap_or_return!(parts[1].to_owned().parse::<IpAddr>());
+                            let asn = unwrap_or_return!(parts[0].to_owned().parse::<u32>(), msg_string).into();
+                            let ip = unwrap_or_return!(parts[1].to_owned().parse::<IpAddr>(), msg_string);
                             (Some(asn), Some(ip))
                         }
                     };
@@ -155,7 +159,7 @@ pub fn parse_ris_live_message(msg_str: &str) -> Result<Vec<BgpElem>, ParserRisli
                             let nexthop = match announcement.next_hop.parse::<IpAddr>(){
                                 Ok(a) => {a}
                                 Err(_) => {
-                                    return Err(ParserRisliveError::IncorrectJson)
+                                    return Err(ParserRisliveError::IncorrectJson(msg_string))
                                 }
                             };
                             for prefix in &announcement.prefixes {
@@ -236,4 +240,14 @@ mod tests {
             println!("{}", elem);
         }
     }
+    #[test]
+    fn test_error_message_3() {
+        let msg_str = r#"
+        {"type": "ris_message","data":{"timestamp":1640553894.84,"peer":"195.66.226.38","peer_asn":"24482","id":"01-2833-11980099","host":"rrc01","type":"UPDATE","path":[24482,30844,328471,328471,328471],"community":[[0,5713],[0,6939],[0,32934],[8714,65010],[8714,65012],[24482,2],[24482,12010],[24482,12011],[24482,65201],[30844,27]],"origin":"igp","aggregator":"4200000002:10.102.100.2","announcements":[{"next_hop":"195.66.224.68","prefixes":["102.66.116.0/24"]}]}}
+        "#;
+        let msg = parse_ris_live_message(&msg_str).unwrap();
+        for elem in msg {
+            println!("{}", elem);
+        }
+    }
 }
